const GBA = @import("gba").GBA;
const Attribute = @import("gba").OAM.Attribute;

/// Square Sprites
pub const square = struct {

    /// 1D-Mapped Square Sprites
    const d1 = struct {};

    /// 2D-Mapped Square Sprites
    pub const d2 = struct {

        /// 2D-Mapped Square Sprites using 4-bit Colour
        pub const bpp4 = struct {
            pub const palette: [8]u32 = [_]u32{
                0x3DEF0000, 0x00005EF7, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
            };

            fn loadPalette() void {
                GBA.memcpy32(GBA.OBJ_PALETTE_RAM, &palette, palette.len * @sizeOf(u32));
            }

            pub const Sprite8x8 = struct {
                const Self = @This();

                const tile = [8]u32{
                    0x21111112, 0x12111121, 0x11211211, 0x11112111, 0x11121111, 0x11211211, 0x12111121, 0x21111112,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color16,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Square,
                        .size = 0,
                    };
                }

                pub fn load() void {
                    // 8x8 Sprites don't differ in any meaningful way between 1D and 2D mapping

                    const tile_width = 8 * 4;
                    const tiles_per_row = 1;

                    const sprite_width = tile_width * tiles_per_row;
                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite16x16 = struct {
                const Self = @This();

                const tile = [32]u32{
                    0x11111112, 0x11111121, 0x11111211, 0x11112111, 0x11121111, 0x11211111, 0x12111111, 0x21111111,
                    0x21111111, 0x12111111, 0x11211111, 0x11121111, 0x11112111, 0x11111211, 0x11111121, 0x11111111,
                    0x11111111, 0x12111111, 0x11211111, 0x11121111, 0x11112111, 0x11111211, 0x11111121, 0x11111112,
                    0x11111112, 0x11111121, 0x11111211, 0x11112111, 0x11121111, 0x11211111, 0x12111111, 0x21111111,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color16,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Square,
                        .size = 1,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1 2
                    //                                       3 4

                    const tile_width = 8 * 4;
                    const tiles_per_row = 2;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x10 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite32x32 = struct {
                const Self = @This();

                const tile = [128]u32{
                    0x11111112, 0x11111121, 0x11111211, 0x11112111, 0x11121111, 0x11211111, 0x12111111, 0x21111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x21111111, 0x12111111, 0x11211111, 0x11121111, 0x11112111, 0x11111211, 0x11111121, 0x11111112,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111112, 0x11111121, 0x11111211, 0x11112111, 0x11121111, 0x11211111, 0x12111111, 0x21111111,
                    0x21111111, 0x12111111, 0x11211111, 0x11121111, 0x11112111, 0x11111211, 0x11111121, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,

                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x12111111, 0x11211111, 0x11121111, 0x11112111, 0x11111211, 0x11111121, 0x11111112,
                    0x11111112, 0x11111121, 0x11111211, 0x11112111, 0x11121111, 0x11211111, 0x12111111, 0x21111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x21111111, 0x12111111, 0x11211111, 0x11121111, 0x11112111, 0x11111211, 0x11111121, 0x11111112,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111112, 0x11111121, 0x11111211, 0x11112111, 0x11121111, 0x11211111, 0x12111111, 0x21111111,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color16,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Square,
                        .size = 2,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1  2  3  4
                    //                                       5  6  7  8
                    //                                       9  10 11 12
                    //                                       13 14 15 16

                    const tile_width = 8 * 4;
                    const tiles_per_row = 4;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x10 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 2), &Self.tile[tile_idx * 2], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 3), &Self.tile[tile_idx * 3], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite64x64 = struct {
                const Self = @This();

                const tile = [512]u32{
                    0x11111112, 0x11111121, 0x11111211, 0x11112111, 0x11121111, 0x11211111, 0x12111111, 0x21111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x21111111, 0x12111111, 0x11211111, 0x11121111, 0x11112111, 0x11111211, 0x11111121, 0x11111112,

                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111112, 0x11111121, 0x11111211, 0x11112111, 0x11121111, 0x11211111, 0x12111111, 0x21111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x21111111, 0x12111111, 0x11211111, 0x11121111, 0x11112111, 0x11111211, 0x11111121, 0x11111112,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,

                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111112, 0x11111121, 0x11111211, 0x11112111, 0x11121111, 0x11211111, 0x12111111, 0x21111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x21111111, 0x12111111, 0x11211111, 0x11121111, 0x11112111, 0x11111211, 0x11111121, 0x11111112,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,

                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111112, 0x11111121, 0x11111211, 0x11112111, 0x11121111, 0x11211111, 0x12111111, 0x21111111,
                    0x21111111, 0x12111111, 0x11211111, 0x11121111, 0x11112111, 0x11111211, 0x11111121, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,

                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x12111111, 0x11211111, 0x11121111, 0x11112111, 0x11111211, 0x11111121, 0x11111112,
                    0x11111112, 0x11111121, 0x11111211, 0x11112111, 0x11121111, 0x11211111, 0x12111111, 0x21111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,

                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x21111111, 0x12111111, 0x11211111, 0x11121111, 0x11112111, 0x11111211, 0x11111121, 0x11111112,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111112, 0x11111121, 0x11111211, 0x11112111, 0x11121111, 0x11211111, 0x12111111, 0x21111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,

                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x21111111, 0x12111111, 0x11211111, 0x11121111, 0x11112111, 0x11111211, 0x11111121, 0x11111112,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111112, 0x11111121, 0x11111211, 0x11112111, 0x11121111, 0x11211111, 0x12111111, 0x21111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,

                    0x21111111, 0x12111111, 0x11211111, 0x11121111, 0x11112111, 0x11111211, 0x11111121, 0x11111112,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111112, 0x11111121, 0x11111211, 0x11112111, 0x11121111, 0x11211111, 0x12111111, 0x21111111,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color16,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Square,
                        .size = 3,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1  2  3  4  5  6  7  8
                    //                                       9  10 11 12 13 14 15 16
                    //                                       17 18 19 20 21 22 23 24
                    //                                       25 26 27 28 29 30 31 32
                    //                                       33 34 35 36 37 38 39 40
                    //                                       41 42 43 44 45 46 47 48
                    //                                       49 50 51 52 53 54 55 56
                    //                                       57 58 59 60 61 62 63 64

                    const tile_width = 8 * 4;
                    const tiles_per_row = 8;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x10 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx * 1], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 2), &Self.tile[tile_idx * 2], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 3), &Self.tile[tile_idx * 3], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 4), &Self.tile[tile_idx * 4], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 5), &Self.tile[tile_idx * 5], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 6), &Self.tile[tile_idx * 6], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 7), &Self.tile[tile_idx * 7], sprite_width);

                    loadPalette();
                }
            };
        };

        /// 2D-Mapped Square Sprites using 8-bit Colour
        pub const bpp8 = struct {
            pub const palette: [8]u32 = [_]u32{
                0x3DEF0000, 0x00005EF7, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
            };

            fn loadPalette() void {
                GBA.memcpy32(GBA.OBJ_PALETTE_RAM, &palette, palette.len * @sizeOf(u32));
            }

            pub const Sprite8x8 = struct {
                const Self = @This();

                const tile = [16]u32{
                    0x01010102, 0x02010101, 0x01010201, 0x01020101, 0x01020101, 0x01010201, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01020101, 0x01010201, 0x01010201, 0x01020101, 0x01010102, 0x02010101,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color256,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Square,
                        .size = 0,
                    };
                }

                pub fn load() void {
                    // 8x8 Sprites don't differ in any meaningful way between 1D and 2D mapping

                    const tile_width = 8 * 8;
                    const tiles_per_row = 1;

                    const sprite_width = tile_width * tiles_per_row;
                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite16x16 = struct {
                const Self = @This();

                const tile = [64]u32{
                    0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101,
                    0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102,
                    0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102,
                    0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101,
                    0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color256,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Square,
                        .size = 1,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1 2
                    //                                       3 4

                    const tile_width = 8 * 8;
                    const tiles_per_row = 2;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x8 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite32x32 = struct {
                const Self = @This();

                const tile = [256]u32{
                    0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102,
                    0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101,
                    0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102,
                    0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102,
                    0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101,
                    0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102,
                    0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color256,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Square,
                        .size = 2,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1  2  3  4
                    //                                       5  6  7  8
                    //                                       9  10 11 12
                    //                                       13 14 15 16

                    const tile_width = 8 * 8;
                    const tiles_per_row = 4;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x8 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 2), &Self.tile[tile_idx * 2], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 3), &Self.tile[tile_idx * 3], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite64x64 = struct {
                const Self = @This();

                const tile = [1024]u32{
                    0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102,
                    0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102,
                    0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102,
                    0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101,

                    0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102,
                    0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102,
                    0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101,

                    0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102,
                    0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102,
                    0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102,
                    0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01020101, 0x01010101, 0x02010101,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color256,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Square,
                        .size = 3,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1  2  3  4  5  6  7  8
                    //                                       9  10 11 12 13 14 15 16
                    //                                       17 18 19 20 21 22 23 24
                    //                                       25 26 27 28 29 30 31 32
                    //                                       33 34 35 36 37 38 39 40
                    //                                       41 42 43 44 45 46 47 48
                    //                                       49 50 51 52 53 54 55 56
                    //                                       57 58 59 60 61 62 63 64

                    const tile_width = 8 * 8;
                    const tiles_per_row = 8;

                    const sprite_width = tile_width * tiles_per_row;
                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * 1, &Self.tile[tile_idx * 1], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * 2, &Self.tile[tile_idx * 2], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * 3, &Self.tile[tile_idx * 3], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * 4, &Self.tile[tile_idx * 4], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * 5, &Self.tile[tile_idx * 5], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * 6, &Self.tile[tile_idx * 6], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * 7, &Self.tile[tile_idx * 7], sprite_width);

                    loadPalette();
                }
            };
        };
    };
};

/// Horizontal Sprites
pub const horizontal = struct {
    /// 1D-Mapped Horizontal Sprites
    const d1 = struct {};

    /// 2D-Mapped Horizontal Sprites
    pub const d2 = struct {

        /// 2D-Mapped Horizontal Sprites using 8-bit Colour
        pub const bpp8 = struct {
            const palette: [8]u32 = [_]u32{
                0x3DEF0000, 0x00006318, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
            };

            fn loadPalette() void {
                GBA.memcpy32(GBA.OBJ_PALETTE_RAM, &palette, palette.len * @sizeOf(u32));
            }

            pub const Sprite16x8 = struct {
                const Self = @This();

                const tile = [32]u32{
                    0x01010202, 0x01010101, 0x02020101, 0x01010101, 0x01010101, 0x01010202, 0x01010101, 0x02020101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010202, 0x02020101, 0x01010101, 0x01010202, 0x01010101,
                    0x01010101, 0x02020101, 0x01010101, 0x01010202, 0x02020101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010202, 0x01010101, 0x02020101, 0x01010101, 0x01010101, 0x01010202, 0x01010101, 0x02020101,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color256,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Horizontal,
                        .size = 0,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1 2

                    const tile_width = 8 * 8;
                    const tiles_per_row = 2;

                    const sprite_width = tile_width * tiles_per_row;
                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite32x8 = struct {
                const Self = @This();

                const tile = [64]u32{
                    0x02020202, 0x01010101, 0x01010101, 0x02020202, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x02020202, 0x02020202, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x02020202, 0x01010101, 0x01010101, 0x02020202,
                    0x01010101, 0x01010101, 0x02020202, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x02020202, 0x01010101, 0x01010101,
                    0x02020202, 0x01010101, 0x01010101, 0x02020202, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02020202, 0x02020202, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x02020202, 0x01010101, 0x01010101, 0x02020202,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color256,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Horizontal,
                        .size = 1,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1 2 3 4

                    const tile_width = 8 * 8;
                    const tiles_per_row = 4;

                    const sprite_width = tile_width * tiles_per_row;
                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite32x16 = struct {
                const Self = @This();

                const tile = [128]u32{
                    0x01010202, 0x01010101, 0x02020101, 0x01010101, 0x01010101, 0x01010202, 0x01010101, 0x02020101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010202, 0x01010101, 0x02020101, 0x01010101, 0x01010101, 0x01010202, 0x01010101, 0x02020101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02020101, 0x01010101, 0x01010202, 0x02020101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02020101, 0x01010101, 0x01010202, 0x02020101, 0x01010101, 0x01010202, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02020101, 0x01010101, 0x01010202, 0x02020101, 0x01010101, 0x01010202, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010202, 0x02020101, 0x01010101, 0x01010202, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010202, 0x01010101, 0x02020101, 0x01010101, 0x01010101, 0x01010202, 0x01010101, 0x02020101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010202, 0x01010101, 0x02020101, 0x01010101, 0x01010101, 0x01010202, 0x01010101, 0x02020101,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color256,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Horizontal,
                        .size = 2,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1 2 3 4
                    //                                       5 6 7 8

                    const tile_width = 8 * 8;
                    const tiles_per_row = 4;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x8 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite64x32 = struct {
                const Self = @This();

                const tile = [512]u32{
                    0x01010202, 0x01010101, 0x02020101, 0x01010101, 0x01010101, 0x01010202, 0x01010101, 0x02020101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010202, 0x01010101, 0x02020101, 0x01010101, 0x01010101, 0x01010202, 0x01010101, 0x02020101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02020101, 0x01010101, 0x01010202, 0x02020101, 0x01010101, 0x01010202, 0x01010101,
                    0x01010101, 0x02020101, 0x01010101, 0x01010202, 0x02020101, 0x01010101, 0x01010202, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010202, 0x01010101, 0x02020101, 0x01010101, 0x01010101, 0x01010202, 0x01010101, 0x02020101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010202, 0x01010101, 0x02020101, 0x01010101, 0x01010101, 0x01010202, 0x01010101, 0x02020101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02020101, 0x01010101, 0x01010202, 0x02020101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02020101, 0x01010101, 0x01010202, 0x02020101, 0x01010101, 0x01010202, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02020101, 0x01010101, 0x01010202, 0x02020101, 0x01010101, 0x01010202, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010202, 0x02020101, 0x01010101, 0x01010202, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010202, 0x01010101, 0x02020101, 0x01010101, 0x01010101, 0x01010202, 0x01010101, 0x02020101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010202, 0x01010101, 0x02020101, 0x01010101, 0x01010101, 0x01010202, 0x01010101, 0x02020101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02020101, 0x01010101, 0x01010202, 0x02020101, 0x01010101, 0x01010202, 0x01010101,
                    0x01010101, 0x02020101, 0x01010101, 0x01010202, 0x02020101, 0x01010101, 0x01010202, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010202, 0x01010101, 0x02020101, 0x01010101, 0x01010101, 0x01010202, 0x01010101, 0x02020101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010202, 0x01010101, 0x02020101, 0x01010101, 0x01010101, 0x01010202, 0x01010101, 0x02020101,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color256,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Horizontal,
                        .size = 3,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1 2  3  4  5  6  7  8
                    //                                       9 10 11 12 13 14 15 16
                    //                                       17 18 19 20 21 22 23 24
                    //                                       25 26 27 28 29 30 31 32

                    const tile_width = 8 * 8;
                    const tiles_per_row = 8;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x8 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx * 1], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 2), &Self.tile[tile_idx * 2], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 3), &Self.tile[tile_idx * 3], sprite_width);

                    loadPalette();
                }
            };
        };

        /// 2D-Mapped Horizontal Sprites using 4-bit Colour
        pub const bpp4 = struct {
            const palette: [8]u32 = [_]u32{
                0x3DEF0000, 0x00006318, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
            };

            fn loadPalette() void {
                GBA.memcpy32(GBA.OBJ_PALETTE_RAM, &palette, palette.len * @sizeOf(u32));
            }

            pub const Sprite16x8 = struct {
                const Self = @This();

                const tile = [16]u32{
                    0x11111122, 0x11112211, 0x11221111, 0x22111111, 0x11111111, 0x11221111, 0x11112211, 0x11111122,
                    0x22111111, 0x11221111, 0x11112211, 0x11111111, 0x11111122, 0x11112211, 0x11221111, 0x22111111,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color16,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Horizontal,
                        .size = 0,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1 2

                    const tile_width = 8 * 4;
                    const tiles_per_row = 2;

                    const sprite_width = tile_width * tiles_per_row;
                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite32x8 = struct {
                const Self = @This();

                const tile = [32]u32{
                    0x11112222, 0x22221111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x22221111, 0x11112222,
                    0x11111111, 0x11111111, 0x11112222, 0x22221111, 0x11111111, 0x11112222, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x22221111, 0x11111111, 0x11112222, 0x22221111, 0x11111111, 0x11111111,
                    0x22221111, 0x11112222, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11112222, 0x22221111,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color16,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Horizontal,
                        .size = 1,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1 2 3 4

                    const tile_width = 8 * 4;
                    const tiles_per_row = 4;

                    const sprite_width = tile_width * tiles_per_row;
                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite32x16 = struct {
                const Self = @This();

                const tile = [64]u32{
                    0x11111122, 0x11112211, 0x11221111, 0x22111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111122, 0x11112211, 0x11221111, 0x22111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x22111111, 0x11221111, 0x11112211, 0x11111111,
                    0x22111111, 0x11221111, 0x11112211, 0x11111122, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x22111111, 0x11221111, 0x11112211, 0x11111122,
                    0x11111111, 0x11221111, 0x11112211, 0x11111122, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111122, 0x11112211, 0x11221111, 0x22111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111122, 0x11112211, 0x11221111, 0x22111111,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color16,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Horizontal,
                        .size = 2,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1 2 3 4
                    //                                       5 6 7 8

                    const tile_width = 8 * 4;
                    const tiles_per_row = 4;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x10 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite64x32 = struct {
                const Self = @This();

                const tile = [256]u32{
                    0x11111122, 0x11112211, 0x11221111, 0x22111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111122, 0x11112211, 0x11221111, 0x22111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x22111111, 0x11221111, 0x11112211, 0x11111122,
                    0x22111111, 0x11221111, 0x11112211, 0x11111122, 0x11111111, 0x11111111, 0x11111111, 0x11111111,

                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111122, 0x11112211, 0x11221111, 0x22111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111122, 0x11112211, 0x11221111, 0x22111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x22111111, 0x11221111, 0x11112211, 0x11111111,
                    0x22111111, 0x11221111, 0x11112211, 0x11111122, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,

                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x22111111, 0x11221111, 0x11112211, 0x11111122,
                    0x11111111, 0x11221111, 0x11112211, 0x11111122, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111122, 0x11112211, 0x11221111, 0x22111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111122, 0x11112211, 0x11221111, 0x22111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,

                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x22111111, 0x11221111, 0x11112211, 0x11111122,
                    0x22111111, 0x11221111, 0x11112211, 0x11111122, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111122, 0x11112211, 0x11221111, 0x22111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111122, 0x11112211, 0x11221111, 0x22111111,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color16,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Horizontal,
                        .size = 3,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1 2  3  4  5  6  7  8
                    //                                       9 10 11 12 13 14 15 16
                    //                                       17 18 19 20 21 22 23 24
                    //                                       25 26 27 28 29 30 31 32

                    const tile_width = 8 * 4;
                    const tiles_per_row = 8;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x10 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx * 1], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 2), &Self.tile[tile_idx * 2], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 3), &Self.tile[tile_idx * 3], sprite_width);

                    loadPalette();
                }
            };
        };
    };
};

/// Vertical Sprites
pub const vertical = struct {
    /// 1D-Mapped Vertical Sprites
    const d1 = struct {};

    /// 2D-Mapped Vertical Sprites
    pub const d2 = struct {

        /// 2D-Mapped Vertical Sprites using 8-bit Colour
        pub const bpp8 = struct {
            const palette: [8]u32 = [_]u32{
                0x3DEF0000, 0x00006318, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
            };

            fn loadPalette() void {
                GBA.memcpy32(GBA.OBJ_PALETTE_RAM, &palette, palette.len * @sizeOf(u32));
            }

            pub const Sprite8x16 = struct {
                const Self = @This();

                const tile = [32]u32{
                    0x01010102, 0x02010101, 0x01010102, 0x02010101, 0x01010201, 0x01020101, 0x01010201, 0x01020101,
                    0x01020101, 0x01010201, 0x01020101, 0x01010201, 0x02010101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010102, 0x01020101, 0x01010201, 0x01020101, 0x01010201,
                    0x01010201, 0x01020101, 0x01010201, 0x01020101, 0x01010102, 0x02010101, 0x01010102, 0x02010101,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color256,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Vertical,
                        .size = 0,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1
                    //                                       2

                    const tile_width = 8 * 8;
                    const tiles_per_row = 1;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x8 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx * 1], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite8x32 = struct {
                const Self = @This();

                const tile = [64]u32{
                    0x01010102, 0x02010101, 0x01010102, 0x02010101, 0x01010102, 0x02010101, 0x01010102, 0x02010101,
                    0x01010201, 0x01020101, 0x01010201, 0x01020101, 0x01010201, 0x01020101, 0x01010201, 0x01020101,
                    0x01020101, 0x01010201, 0x01020101, 0x01010201, 0x01020101, 0x01010201, 0x01020101, 0x01010201,
                    0x02010101, 0x01010101, 0x02010101, 0x01010101, 0x02010101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010102, 0x01010101, 0x01010102, 0x01010101, 0x01010102,
                    0x01020101, 0x01010201, 0x01020101, 0x01010201, 0x01020101, 0x01010201, 0x01020101, 0x01010201,
                    0x01010201, 0x01020101, 0x01010201, 0x01020101, 0x01010201, 0x01020101, 0x01010201, 0x01020101,
                    0x01010102, 0x02010101, 0x01010102, 0x02010101, 0x01010102, 0x02010101, 0x01010102, 0x02010101,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color256,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Vertical,
                        .size = 1,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1
                    //                                       2
                    //                                       3
                    //                                       4

                    const tile_width = 8 * 8;
                    const tiles_per_row = 1;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x8 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx * 1], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 2), &Self.tile[tile_idx * 2], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 3), &Self.tile[tile_idx * 3], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite16x32 = struct {
                const Self = @This();

                const tile = [128]u32{
                    0x01010102, 0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01010201, 0x01010101,
                    0x01020101, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x02010101, 0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01020101,
                    0x01010101, 0x01010201, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101, 0x01010102,
                    0x01010101, 0x01010102, 0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01010201,
                    0x01010101, 0x01020101, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101, 0x02010101,
                    0x02010101, 0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01020101, 0x01010101,
                    0x01010201, 0x01010101, 0x01010201, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01020101, 0x01010101, 0x01020101,
                    0x01010101, 0x01010201, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101, 0x01010102,
                    0x01010102, 0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01010201, 0x01010101,
                    0x01020101, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101, 0x02010101, 0x01010101,
                    0x02010101, 0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01020101, 0x01010101,
                    0x01010201, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101, 0x01010102, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01010201,
                    0x01010101, 0x01020101, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101, 0x02010101,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color256,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Vertical,
                        .size = 2,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1  2
                    //                                       3  4
                    //                                       5  6
                    //                                       7  8

                    const tile_width = 8 * 8;
                    const tiles_per_row = 2;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x8 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx * 1], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 2), &Self.tile[tile_idx * 2], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 3), &Self.tile[tile_idx * 3], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite32x64 = struct {
                const Self = @This();

                const tile = [512]u32{
                    0x01010102, 0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01010201, 0x01010101,
                    0x01020101, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x02010101, 0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01020101,
                    0x01010101, 0x01010201, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101, 0x01010102,

                    0x01010101, 0x01010102, 0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01010201,
                    0x01010101, 0x01020101, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101, 0x02010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x02010101, 0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01020101, 0x01010101,
                    0x01010201, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101, 0x01010102, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010102, 0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01010201, 0x01010101,
                    0x01020101, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x02010101, 0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01020101,
                    0x01010101, 0x01010201, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101, 0x01010102,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01010201,
                    0x01010101, 0x01020101, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101, 0x02010101,
                    0x02010101, 0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01020101, 0x01010101,
                    0x01010201, 0x01010101, 0x01010201, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01020101, 0x01010101, 0x01020101,
                    0x01010101, 0x01010201, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101, 0x01010102,
                    0x01010102, 0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01010201, 0x01010101,
                    0x01020101, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101, 0x02010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x02010101, 0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01020101, 0x01010101,
                    0x01010201, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101, 0x01010102, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01010201,
                    0x01010101, 0x01020101, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101, 0x02010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,

                    0x01010101, 0x02010101, 0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01020101,
                    0x01010101, 0x01010201, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101, 0x01010102,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010102, 0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01010201, 0x01010101,
                    0x01020101, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101, 0x02010101, 0x01010101,

                    0x02010101, 0x01010101, 0x02010101, 0x01010101, 0x01020101, 0x01010101, 0x01020101, 0x01010101,
                    0x01010201, 0x01010101, 0x01010201, 0x01010101, 0x01010102, 0x01010101, 0x01010102, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101, 0x01010101,
                    0x01010101, 0x01010102, 0x01010101, 0x01010102, 0x01010101, 0x01010201, 0x01010101, 0x01010201,
                    0x01010101, 0x01020101, 0x01010101, 0x01020101, 0x01010101, 0x02010101, 0x01010101, 0x02010101,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color256,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Vertical,
                        .size = 3,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1  2  3  4
                    //                                       5  6  7  8
                    //                                       9  10 11 12
                    //                                       13 14 15 16
                    //                                       17 18 19 20
                    //                                       21 22 23 24
                    //                                       25 26 27 28
                    //                                       29 30 31 32

                    const tile_width = 8 * 8;
                    const tiles_per_row = 4;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x8 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx * 1], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 2), &Self.tile[tile_idx * 2], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 3), &Self.tile[tile_idx * 3], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 4), &Self.tile[tile_idx * 4], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 5), &Self.tile[tile_idx * 5], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 6), &Self.tile[tile_idx * 6], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 7), &Self.tile[tile_idx * 7], sprite_width);

                    loadPalette();
                }
            };
        };

        /// 2D-Mapped Vertical Sprites using 4-bit Colour
        pub const bpp4 = struct {
            pub const palette: [8]u32 = [_]u32{
                0x3DEF0000, 0x00006318, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
            };

            fn loadPalette() void {
                GBA.memcpy32(GBA.OBJ_PALETTE_RAM, &palette, palette.len * @sizeOf(u32));
            }

            pub const Sprite8x16 = struct {
                const Self = @This();

                const tile = [16]u32{
                    0x21111112, 0x21111112, 0x12111121, 0x12111121, 0x11211211, 0x11211211, 0x11112111, 0x11112111,
                    0x11121111, 0x11121111, 0x11211211, 0x11211211, 0x12111121, 0x12111121, 0x21111112, 0x21111112,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color16,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Vertical,
                        .size = 0,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1
                    //                                       2

                    const tile_width = 8 * 4;
                    const tiles_per_row = 1;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x10 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx * 1], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite8x32 = struct {
                const Self = @This();

                const tile = [32]u32{
                    0x21111112, 0x21111112, 0x21111112, 0x21111112, 0x12111121, 0x12111121, 0x12111121, 0x12111121,
                    0x11211211, 0x11211211, 0x11211211, 0x11211211, 0x11112111, 0x11112111, 0x11112111, 0x11112111,
                    0x11121111, 0x11121111, 0x11121111, 0x11121111, 0x11211211, 0x11211211, 0x11211211, 0x11211211,
                    0x12111121, 0x12111121, 0x12111121, 0x12111121, 0x21111112, 0x21111112, 0x21111112, 0x21111112,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color16,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Vertical,
                        .size = 1,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1
                    //                                       2
                    //                                       3
                    //                                       4

                    const tile_width = 8 * 4;
                    const tiles_per_row = 1;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x10 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx * 1], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 2), &Self.tile[tile_idx * 2], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 3), &Self.tile[tile_idx * 3], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite16x32 = struct {
                const Self = @This();

                const tile = [64]u32{
                    0x11111112, 0x11111112, 0x11111121, 0x11111121, 0x11111211, 0x11111211, 0x11112111, 0x11112111,
                    0x21111111, 0x21111111, 0x12111111, 0x12111111, 0x11211111, 0x11211111, 0x11121111, 0x11121111,
                    0x11121111, 0x11121111, 0x11211111, 0x11211111, 0x12111111, 0x12111111, 0x21111111, 0x21111111,
                    0x11112111, 0x11112111, 0x11111211, 0x11111211, 0x11111121, 0x11111121, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x12111111, 0x12111111, 0x11211111, 0x11211111, 0x11121111, 0x11121111,
                    0x11111112, 0x11111112, 0x11111121, 0x11111121, 0x11111211, 0x11111211, 0x11112111, 0x11112111,
                    0x11112111, 0x11112111, 0x11111211, 0x11111211, 0x11111121, 0x11111121, 0x11111112, 0x11111112,
                    0x11121111, 0x11121111, 0x11211111, 0x11211111, 0x12111111, 0x12111111, 0x21111111, 0x21111111,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color16,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Vertical,
                        .size = 2,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1  2
                    //                                       3  4
                    //                                       5  6
                    //                                       7  8

                    const tile_width = 8 * 4;
                    const tiles_per_row = 2;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x10 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx * 1], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 2), &Self.tile[tile_idx * 2], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 3), &Self.tile[tile_idx * 3], sprite_width);

                    loadPalette();
                }
            };

            pub const Sprite32x64 = struct {
                const Self = @This();

                const tile = [256]u32{
                    0x11111112, 0x11111112, 0x11111121, 0x11111121, 0x11111211, 0x11111211, 0x11112111, 0x11112111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x21111111, 0x21111111, 0x12111111, 0x12111111, 0x11211111, 0x11211111, 0x11121111, 0x11121111,
                    0x11121111, 0x11121111, 0x11211111, 0x11211111, 0x12111111, 0x12111111, 0x21111111, 0x21111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11112111, 0x11112111, 0x11111211, 0x11111211, 0x11111121, 0x11111121, 0x11111112, 0x11111112,

                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111112, 0x11111112, 0x11111121, 0x11111121, 0x11111211, 0x11111211, 0x11112111, 0x11112111,
                    0x21111111, 0x21111111, 0x12111111, 0x12111111, 0x11211111, 0x11211111, 0x11121111, 0x11121111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11121111, 0x11121111, 0x11211111, 0x11211111, 0x12111111, 0x12111111, 0x21111111, 0x21111111,
                    0x11112111, 0x11112111, 0x11111211, 0x11111211, 0x11111121, 0x11111121, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,

                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x12111111, 0x12111111, 0x11211111, 0x11211111, 0x11121111, 0x11121111,
                    0x11111112, 0x11111112, 0x11111121, 0x11111121, 0x11111211, 0x11111211, 0x11112111, 0x11112111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11112111, 0x11112111, 0x11111211, 0x11111211, 0x11111121, 0x11111121, 0x11111112, 0x11111112,
                    0x11121111, 0x11121111, 0x11211111, 0x11211111, 0x12111111, 0x12111111, 0x21111111, 0x21111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,

                    0x21111111, 0x21111111, 0x12111111, 0x12111111, 0x11211111, 0x11211111, 0x11121111, 0x11121111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111112, 0x11111112, 0x11111121, 0x11111121, 0x11111211, 0x11111211, 0x11112111, 0x11112111,
                    0x11112111, 0x11112111, 0x11111211, 0x11111211, 0x11111121, 0x11111121, 0x11111112, 0x11111112,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111, 0x11111111,
                    0x11121111, 0x11121111, 0x11211111, 0x11211111, 0x12111111, 0x12111111, 0x21111111, 0x21111111,
                };

                pub fn getAttribute() Attribute {
                    return Attribute{
                        .paletteMode = .Color16,
                        .doubleSizeOrVisible = false,
                        .palette = 0,
                        .tileIndex = 0,
                        .shape = .Vertical,
                        .size = 3,
                    };
                }

                pub fn load() void {
                    // In Memory, Tile Map is laid out like: 1  2  3  4
                    //                                       5  6  7  8
                    //                                       9  10 11 12
                    //                                       13 14 15 16
                    //                                       17 18 19 20
                    //                                       21 22 23 24
                    //                                       25 26 27 28
                    //                                       29 30 31 32

                    const tile_width = 8 * 4;
                    const tiles_per_row = 4;

                    const sprite_width = tile_width * tiles_per_row;
                    const offset = 0x10 / tiles_per_row;

                    const tile_idx = sprite_width / @sizeOf(u32);

                    GBA.memcpy32(GBA.SPRITE_VRAM, &Self.tile[0], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 1), &Self.tile[tile_idx * 1], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 2), &Self.tile[tile_idx * 2], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 3), &Self.tile[tile_idx * 3], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 4), &Self.tile[tile_idx * 4], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 5), &Self.tile[tile_idx * 5], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 6), &Self.tile[tile_idx * 6], sprite_width);
                    GBA.memcpy32(GBA.SPRITE_VRAM + sprite_width * (offset * 7), &Self.tile[tile_idx * 7], sprite_width);

                    loadPalette();
                }
            };
        };
    };
};
